// ============================================================================
// ==                                                                        ==
// == AMDiS - Adaptive multidimensional simulations                          ==
// ==                                                                        ==
// ==  http://www.amdis-fem.org                                              ==
// ==                                                                        ==
// ============================================================================
//
// Software License for AMDiS
//
// Copyright (c) 2010 Dresden University of Technology 
// All rights reserved.
// Authors: Simon Vey, Thomas Witkowski et al.
//
// This file is part of AMDiS
//
// See also license.opensource.txt in the distribution.



/** \file ITL_OEMSolver.h */


#ifndef AMDIS_ITL_OEM_SOLVER_H
#define AMDIS_ITL_OEM_SOLVER_H

#include "MTL4Solver.h"
#include "ITL_OEMSolver.hh"
#include "MTL4Types.h"

#include <boost/numeric/itl/itl.hpp>
#include <boost/numeric/mtl/mtl.hpp>

namespace AMDiS {

  /**
   * \ingroup Solver
   * 
   *\brief
   * Template for using solvers from ITL in AMDiS.
   * Uses a previous definition of MTLMatrix and MTLVector.
   * Additional template generalization of ITL_OEMSolver destroys class hierarchy 
   */

  template <typename ITLSolver>
  class ITL_OEMSolver : public MTL4Solver< MTLTypes::MTLMatrix, MTLTypes::MTLVector, ITL_OEMSolver_runner< ITLSolver, MTLTypes::MTLMatrix, MTLTypes::MTLVector> >  {

  protected:
  public:
    /// The constructor reads needed parameters and sets solvers \ref name.
    ITL_OEMSolver(std::string name) : 
      MTL4Solver< MTLTypes::MTLMatrix, MTLTypes::MTLVector, ITL_OEMSolver_runner< ITLSolver, MTLTypes::MTLMatrix, MTLTypes::MTLVector > >(name)  {}

  
    ~ITL_OEMSolver() {}

    /// Creator class used in the OEMSolverMap.
    class Creator : public OEMSolverCreator
    {
    public:
      virtual ~Creator() {}
      
      /// Returns a new CGSolver object.
      OEMSolver* create() 
      { 
	return new ITL_OEMSolver<ITLSolver>(this->name); 
      }
    };
  
  };

  /**
   * \ingroup Solver
   * 
   *\brief
   * Template for using solvers from ITL that have one run-time parameter in AMDiS. 
   */
  template <typename ITLSolver>
  class ITL_OEMSolver_para
    : public MTL4Solver< MTLMatrix, MTLVector, ITL_OEMSolver_para_runner< ITLSolver, MTLTypes::MTLMatrix, MTLTypes::MTLVector > > // ITL_OEMSolver<ITLSolver>
  {
  public:
    /// The constructor reads needed parameters and sets solvers \ref name.
    ITL_OEMSolver_para(std::string name) 
      : MTL4Solver< MTLMatrix, MTLVector, ITL_OEMSolver_para_runner< ITLSolver, MTLTypes::MTLMatrix, MTLTypes::MTLVector > >(name) 
    {

    }
      
    ~ITL_OEMSolver_para() {}

    /// Set parameter of iterative solver

    class Creator : public OEMSolverCreator
    {
    public:
      virtual ~Creator() {}
      
      /// Returns a new CGSolver object.
      OEMSolver* create() 
      { 
	return new ITL_OEMSolver_para<ITLSolver>(this->name); 
      }
    }; 
      
  private:
  };
} // namespace AMDiS

#endif // AMDIS_ITL_OEM_SOLVER_H
